%{
// FILE: arithmetic.lex
// Purpose: A simple first flex program to scan for the tokens
// of an arithmetic expression with: + - * / ** parentheses numbers.
// To compile and run this first flex program, give the commands:
//   flex -t arithmetic.lex >arithmeticlex.cxx
//   g++ -DFLEX_DEBUG arithmeticlex.cxx -ll -ly -lm -o scantest
// I'll discuss the meaning of these commands in class. The four
// commands together create an executable file called scantest.
// When you run the file, it will read an arithmetic expression 
// from standard input and print a list of the tokens in the
// expression.

// Provide cin and cout:
#include <iostream.h> 

// Define all the token numbers. Note that the scanner created by
// flex will automatically return 0 at end-of-file (not -1).
// Two symbols (open and close parentheses) will just use their
// ascii values for the token number, so no definition is
// needed.
#ifdef FLEX_DEBUG
    #define END     0	    
    #define	NUMBER	258
    #define	PLUS	259
    #define	MINUS	260
    #define	TIMES	261
    #define	DIVIDE	262
    #define EXP     263
#else
    #include "arithmetic.tab.h"
#endif

// Declare a global variable to keep track of the current input 
// line number:
int yyline = 1;

%}


%{
// DEFINITIONS OF REGULAR EXPRESSIONS
%}
separator         [ \n\r\t\v]
digits            [0-9]+
exponent          [eE][+-]?{digits}
number1           {digits}{exponent}?
number2           {digits}"."{digits}{exponent}?            
number3           {digits}"."{exponent}?            
number4           "."{digits}{exponent}?
number            {number1}|{number2}|{number3}|{number4} 
%%

%{
// FLEX RULES.
// Notes:
//  1. yytext is a null-terminated string that contains the characters
//  of the pattern that was just matched. For example, yytext[0]
//  is the first character of the pattern that was matched.
//
//  2. The symbol | instead of an action means "or" and the pattern
//  is combined with the next pattern (with the same action when
//  they are matched). I use this "or" operation for the 
//  open and close parentheses since in both cases the token
//  number is just the ascii value of the character (which is
//  obtained from yytext[0]).
//
//  3. If you want the action to contain more than one line,
//  the first action line should be an open bracket, and put a
//  close bracket after the last action line.
%}
{number}            return NUMBER;
"+"                 return PLUS;
"-"                 return MINUS;
"*"                 return TIMES;
"/"                 return DIVIDE;
"**"                return EXP;

"("                 |
")"                 return yytext[0];

{separator}         {
                        if (yytext[0] == '\n')
                            yyline++;
                    }
.                   return yytext[0];

%%
/* USER SUBROUTINES SECTION */
#ifdef FLEX_DEBUG
    int main( )
    {
        int next_token;
        while ((next_token = yylex( )) != END)
        {
            switch (next_token)
            {
                case NUMBER:    cout << "NUMBER " << yytext << endl;
		                break;
                case PLUS:      cout << "PLUS" << endl;
                                break;
                case MINUS:     cout << "MINUS" << endl;
                                break;
                case TIMES:     cout << "TIMES" << endl;
                                break;
                case DIVIDE:    cout << "DIVIDE" << endl;
                                break;
                case EXP:       cout << "EXP" << endl;
                                break;
                case '(':       cout << "Open Parenthesis" << endl;
                                break;
                case ')':       cout << "Close Parenthesis" << endl;
                                break;
                default:        cout << "UNKNOWN TOKEN " << next_token;
                                cout << endl;
            }
        }
        cout << "END OF FILE..." << yyline-1 << " lines were read." << endl;
    }
#endif
